/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal;

import java.util.StringTokenizer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
//import org.eclipse.ui.internal.misc.CheckboxTreeAndListGroup;
//import sun.security.krb5.internal.i;

import fr.improve.csharp.ui.internal.editor.CSharpCompilerAction;
import fr.improve.csharp.ui.internal.editor.utils.RelativePath;

/**
 * Property page to set up compile-time properties for c# files.
 * <br>
 * 
 * @see PropertyPage
 */

public class CSharpPropertyPage extends PropertyPage {

	/*
	 * Qualified names for file properties
	 */
 	private static final String PREFIX = "c-sharp_property_page_";
 	
	public static final QualifiedName outputNameQName = new QualifiedName(PREFIX, "output_name");
	public static final QualifiedName targetQName = new QualifiedName(PREFIX, "target_name");
	public static final QualifiedName docFileNameQName = new QualifiedName(PREFIX, "doc_file_name");
	public static final QualifiedName refFileTextQName = new QualifiedName(PREFIX, "referenced_files_text");
	public static final QualifiedName refFileListQName = new QualifiedName(PREFIX, "referenced_files_list");
	public static final QualifiedName moduleTextQName = new QualifiedName(PREFIX, "modules_added_text");
	public static final QualifiedName moduleListQName = new QualifiedName(PREFIX, "modules_added_list");
	public static final QualifiedName otherArgsQName = new QualifiedName(PREFIX, "other_arguments");
	
	/*
	 * The widgets which persistent properties are taken from
	 */
	protected Text outputNameText;
	protected Combo targetCombo;
	protected Text docFileNameText;
	protected Text refFileNameText;
	protected Text moduleNameText;
	protected Text otherArgsText;
	

	/**
	 * Max value width in characters before wrapping
	 */
	private static final int MAX_NAME_VALUE_WIDTH = 30;

	/**
	 * Max value width in file name list before wrapping
	 */
	private static final int MAX_LIST_VALUE_WIDTH = 60;
	

	/**
	 * The constructor.
	 */
	public CSharpPropertyPage() {
	}

	/**
	 * @see PropertyPage#createContents
	 */
	protected Control createContents(Composite parent) {

		IResource resource = (IResource) getElement();

		// create the Control
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL);
		data.grabExcessHorizontalSpace = true;
		composite.setLayoutData(data);

		// create the output arguments panel
		createOutputGroup(composite, resource);

		// create the intput arguments panel
		createInputGroup(composite, resource);
		
		// create the other arguments panel
		createOtherArgsGroup(composite, resource);

		return composite;
	}

	private void createOutputGroup(Composite parent, IResource resource) {

		Group outputComposite = new Group(parent, SWT.TITLE);
		outputComposite.setText("Output parameters");
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		outputComposite.setLayout(layout);
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		outputComposite.setLayoutData(data);

		//---------------------- OUPUT FILE NAME ---------------------------
		//The output name label
		Label outputNameLabel = new Label(outputComposite, SWT.NONE);
		outputNameLabel.setText("Output name: ");
		GridData gd = new GridData();
		outputNameLabel.setLayoutData(gd);

		//The input text for the output name
		outputNameText = new Text(outputComposite, SWT.WRAP | SWT.BORDER);
		String outputName = null;
		try {
			outputName = resource.getPersistentProperty(outputNameQName);	
		} catch (CoreException e) {
			// just ignore
		}
		if ( outputName != null)
			outputNameText.setText(outputName);
		gd = new GridData();
		gd.widthHint = convertWidthInCharsToPixels(MAX_NAME_VALUE_WIDTH);
		outputNameText.setLayoutData(gd);

		//---------------------------- TARGET ------------------------------
		//The target label
		Label targetLabel = new Label(outputComposite, SWT.LEFT);
		targetLabel.setText("Target: ");

		//The combo for choosing the target
		targetCombo = new Combo(outputComposite, SWT.NONE);
		targetCombo.add("DOS executable (.exe)");
		targetCombo.add("Windows executable (.winexe)");
		targetCombo.add("Library (.dll)");
		targetCombo.add("Module (.netmodule)");
		String kindOfTarget = null;
		try {
			kindOfTarget = resource.getPersistentProperty(targetQName);
		} catch (CoreException e) {
		}
		int selectedItem = 0;
		if (kindOfTarget != null && !kindOfTarget.equals(CSharpCompilerAction.EXE)) {
			if (kindOfTarget.equals(CSharpCompilerAction.WINEXE))
				selectedItem = 1;
			else if (kindOfTarget.equals(CSharpCompilerAction.LIBRARY))
				selectedItem = 2;
			else if (kindOfTarget.equals(CSharpCompilerAction.MODULE))
				selectedItem = 3;
		}
		targetCombo.select(selectedItem);

		//---------------------- DOC FILE NAME ------------------------------
		//The doc label
		Label docLabel = new Label(outputComposite, SWT.NONE);
		docLabel.setText("Documentation file name: ");
		gd = new GridData();
		docLabel.setLayoutData(gd);

		//The input text for the doc file name
		docFileNameText = new Text(outputComposite, SWT.WRAP | SWT.BORDER);
		String docFileName = null;
		try {
			docFileName = resource.getPersistentProperty(docFileNameQName);	
		} catch (CoreException e) {
			// just ignore
		}
		if ( docFileName != null)
			docFileNameText.setText(docFileName);
		gd = new GridData();
		gd.widthHint = convertWidthInCharsToPixels(MAX_NAME_VALUE_WIDTH);
		docFileNameText.setLayoutData(gd);

	}

	private void createInputGroup(Composite parent, IResource res) {

		final IResource resource = res;

		Group outputComposite = new Group(parent, SWT.TITLE);
		outputComposite.setText("Input parameters");
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		outputComposite.setLayout(layout);
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		outputComposite.setLayoutData(data);

		//---------------- ASSEMBLY FILES TO REFERENCE ----------------------
		//The files to refer to
		Label filesLabel = new Label(outputComposite, SWT.NONE);
		filesLabel.setText("files to refer to: ");
		GridData gd = new GridData();
		filesLabel.setLayoutData(gd);

		//The input text for the file names
		refFileNameText = new Text(outputComposite, SWT.WRAP | SWT.BORDER);
		refFileNameText.setEditable(false);
		String fileNamesForText = null;
		try {
			fileNamesForText = resource.getPersistentProperty(refFileTextQName);	
		} catch (CoreException e) {
			// just ignore
		}
		if ( fileNamesForText != null)
			refFileNameText.setText(fileNamesForText);		
		gd = new GridData();
		gd.widthHint = convertWidthInCharsToPixels(MAX_LIST_VALUE_WIDTH);
		refFileNameText.setLayoutData(gd);

		//The button to launch the file selction dialog
		Button fileButton = new Button(outputComposite, SWT.NONE);
		fileButton.setText("Select file(s) ");
		fileButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				ResourceSelectionDialog dialog =
					new ResourceSelectionDialog(
						getShell(),
						resource.getProject(),
						"Choose the assembly files you want to reference:");

				// sets the file previously selected
				String fileNames = null;
				try {
					fileNames = resource.getPersistentProperty(refFileListQName);
				} catch (CoreException ex) {
					// if there's an error, do nothing
				}
				if (fileNames != null)
					dialog.setInitialSelections(retrieveFilesFromPeristentProperty(fileNames));

				dialog.open();
				Object[] list = dialog.getResult();

				if (list != null) {
					// saves the list as a persistent property for the file
					try {
						resource.setPersistentProperty(
							refFileListQName,
							convertToSemiColumnSeparatedListOfFullPath(list));
					} catch (CoreException ex) {
						// silently ignore the exception
					}

					// and prints out in the text field the file name list
					refFileNameText.setText(
						convertToSemiColumnSeparatedListOfRelativePath(list, resource));
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		fileButton.setLayoutData(gd);
		
		//The button to reset the field
		Button refResetButton = new Button(outputComposite, SWT.NONE);
		refResetButton.setText("Reset");
		refResetButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				refFileNameText.setText("");
				try {
					resource.setPersistentProperty(refFileListQName,"");
				} catch (CoreException ex) {
					// silently ignore the exception
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		gd = new GridData();
		refResetButton.setLayoutData(gd);
		

		//---------------------- MODULES TO ADD -----------------------------
		//The modules to add
		Label modulesLabel = new Label(outputComposite, SWT.NONE);
		modulesLabel.setText("modules to add: ");
		gd = new GridData();
		modulesLabel.setLayoutData(gd);

		//The input text for the module names
		moduleNameText = new Text(outputComposite, SWT.WRAP | SWT.BORDER);
		moduleNameText.setEditable(false);
		String moduleNamesForText = null;
		try {
			moduleNamesForText = resource.getPersistentProperty(moduleTextQName);	
		} catch (CoreException e) {
			// just ignore
		}
		if ( moduleNamesForText != null)
			moduleNameText.setText(moduleNamesForText);
		gd = new GridData();
		gd.widthHint = convertWidthInCharsToPixels(MAX_LIST_VALUE_WIDTH);
		moduleNameText.setLayoutData(gd);

		//The button to launch the file selction dialog
		Button moduleButton = new Button(outputComposite, SWT.NONE);
		moduleButton.setText("Select module(s) ");
		moduleButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				ResourceSelectionDialog dialog =
					new ResourceSelectionDialog(
						getShell(),
						resource.getProject(),
						"Choose the modules you want to add:");

				// sets the file previously selected
				String moduleNames = null;
				try {
					moduleNames = resource.getPersistentProperty(moduleListQName);
				} catch (CoreException ex) {
					// if there's an error, do nothing
				}
				if (moduleNames != null)
					dialog.setInitialSelections(retrieveFilesFromPeristentProperty(moduleNames));

				dialog.open();
				Object[] list = dialog.getResult();

				if (list != null) {
					// saves the list as a persistent property for the file
					try {
						resource.setPersistentProperty(
							moduleListQName,
							convertToSemiColumnSeparatedListOfFullPath(list));
					} catch (CoreException ex) {
						// silently ignore the exception
					}

					// and prints out in the text field the file name list
					moduleNameText.setText(
						convertToSemiColumnSeparatedListOfRelativePath(list, resource));
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		moduleButton.setLayoutData(gd);
		
		//The button to reset the field
		Button moduleResetButton = new Button(outputComposite, SWT.NONE);
		moduleResetButton.setText("Reset");
		moduleResetButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				moduleNameText.setText("");
				try {
					resource.setPersistentProperty(moduleListQName,"");
				} catch (CoreException ex) {
					// silently ignore the exception
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		gd = new GridData();
		moduleResetButton.setLayoutData(gd);

	}
	

	private void createOtherArgsGroup(Composite parent, IResource resource) {
		Group outputComposite = new Group(parent, SWT.TITLE);
		outputComposite.setText("Others arguments");
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		outputComposite.setLayout(layout);
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		outputComposite.setLayoutData(data);

		//The label
		Label label = new Label(outputComposite, SWT.NONE);
		label.setText("Other command-line arguments: ");
		GridData gd = new GridData();
		label.setLayoutData(gd);

		//The input text
		otherArgsText = new Text(outputComposite, SWT.WRAP | SWT.BORDER);
		String args = null;
		try {
			args = resource.getPersistentProperty(otherArgsQName);	
		} catch (CoreException e) {
			// just ignore
		}
		if ( args != null)
			otherArgsText.setText(args);
		gd = new GridData();
		gd.widthHint = convertWidthInCharsToPixels(MAX_LIST_VALUE_WIDTH);
		otherArgsText.setLayoutData(gd);

	}

	/**
	 * Creates a semi-column separated list of relative paths for the given String array
	 */
	protected String convertToSemiColumnSeparatedListOfRelativePath(
		Object[] files,
		IResource sourceCodeFile) {
		StringBuffer names = new StringBuffer();
		for (int i = 0; i < files.length; i++) {
			names.append(
				RelativePath.computeRelativePath((IFile) files[i], sourceCodeFile));

			// adds the semi column, except if this is the last file
			if (i < files.length - 1)
				names.append(";");
		}
		return names.toString();
	}

	/**
	 * Creates a semi-column separated list of full paths for the given String array
	 */
	protected String convertToSemiColumnSeparatedListOfFullPath(Object[] files) {
		StringBuffer names = new StringBuffer();
		for (int i = 0; i < files.length; i++) {
			names.append(((IFile) files[i]).getFullPath());
			// adds the semi column, except if this is the last file
			if (i < files.length - 1)
				names.append(";");
		}
		return names.toString();
	}

	/**
	 * Creates a list of IFile objects from a persistent property String
	 */
	protected Object[] retrieveFilesFromPeristentProperty(String list) {
		StringTokenizer tokenizer = new StringTokenizer(list, ";");
		Object[] result = new Object[tokenizer.countTokens()];
		int i = 0;
		while (tokenizer.hasMoreElements()) {
			result[i] =
				ResourcesPlugin.getWorkspace().getRoot().getFile(
					new Path((String) tokenizer.nextElement()));
			i++;
		}
		return result;
	}
	
	/**
	 * Overrides the perfomOk method to store the different file persistent
	 * properties.
	 * 
	 * @see PreferencePage#performOk()
	 */
	public boolean performOk() {
		IResource resource = (IResource) getElement();
		
		// stores the persistent properties
		try {
			resource.setPersistentProperty(outputNameQName, outputNameText.getText());
		} catch (CoreException e) {}
		
		try {
			String kindOfTarget = null;
			switch (targetCombo.getSelectionIndex()) {
				case 0 :
					kindOfTarget = CSharpCompilerAction.EXE;
					break;
				case 1 :
					kindOfTarget = CSharpCompilerAction.WINEXE;
					break;
				case 2 :
					kindOfTarget = CSharpCompilerAction.LIBRARY;
					break;
				case 3 :
					kindOfTarget = CSharpCompilerAction.MODULE;
					break;
				default :
					kindOfTarget = CSharpCompilerAction.EXE;
					break;
			}
			resource.setPersistentProperty(targetQName, kindOfTarget);
		} catch (CoreException e) {}
		
		try {
			resource.setPersistentProperty(docFileNameQName, docFileNameText.getText());
		} catch (CoreException e) {}
		
		try {
			resource.setPersistentProperty(refFileTextQName, refFileNameText.getText());
		} catch (CoreException e) {}
		
		try {
			resource.setPersistentProperty(moduleTextQName, moduleNameText.getText());
		} catch (CoreException e) {}
		
		try {
			resource.setPersistentProperty(otherArgsQName, otherArgsText.getText());
		} catch (CoreException e) {}
		
		
		return super.performOk();
	}

}