/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;


import java.util.ResourceBundle;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.texteditor.BasicTextEditorActionContributor;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.RetargetTextEditorAction;
import org.eclipse.ui.texteditor.TextEditorAction;

/**
 * Contributes interesting CSharp actions to the desktop's Edit menu and
 * the toolbar.
 */
public class CSharpActionContributor extends BasicTextEditorActionContributor {

	protected RetargetTextEditorAction contentAssistProposalAction;
	protected CSharpCompilerAction compilerAction;

	/**
	 * Default constructor.
	 */
	public CSharpActionContributor() {
		super();

		contentAssistProposalAction= new RetargetTextEditorAction(NLS.getResourceBundle(), "ContentAssistProposal."); //$NON-NLS-1$
		compilerAction = CSharpCompilerAction.getInstance();
	}
	
	/* (non-Javadoc)
	 * Method declared on EditorActionBarContributor
	 */
	public void contributeToMenu(IMenuManager menuManager) {
		IMenuManager editMenu= menuManager.findMenuUsingPath(IWorkbenchActionConstants.M_EDIT);
		if (editMenu != null) {
			editMenu.add(new Separator());
			editMenu.add(contentAssistProposalAction);
		}
	}
	
	/* (non-Javadoc)
	 * Method declared on EditorActionBarContributor
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager) {
		toolBarManager.add(new Separator());
		toolBarManager.add(compilerAction);
	}
	
	/* (non-Javadoc)
	 * Method declared on EditorActionBarContributor
	 */
	public void setActiveEditor(IEditorPart part) {
		super.setActiveEditor(part);

		ITextEditor editor= null;
		if (part instanceof ITextEditor)
			editor= (ITextEditor) part;

		contentAssistProposalAction.setAction(getAction(editor, "ContentAssistProposal")); //$NON-NLS-1$
		
		compilerAction.setEditor(editor);
		compilerAction.update();
	}
}
