/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;

import java.io.*;
import java.util.*;

import fr.improve.csharp.ui.internal.CSharpPropertyPage;
import fr.improve.csharp.ui.internal.CSharpUIPlugin;
import fr.improve.csharp.ui.internal.ICSharpPreferenceConstants;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.viewers.*;
import org.eclipse.ui.*;
import org.eclipse.ui.part.*;
import org.eclipse.ui.console.*;
import org.eclipse.ui.texteditor.*;
import org.eclipse.ui.views.tasklist.TaskList;

//import tinyOS.nesc12.collector.a.iF;

/**
 * Class that defines the action of compiling the current C Sharp file
 */

public class CSharpCompilerAction extends TextEditorAction {

	private MessageConsole console;
	public static final String EXE = "exe"; //$NON-NLS-1$
	public static final String WINEXE = "winexe"; //$NON-NLS-1$
	public static final String LIBRARY = "library"; //$NON-NLS-1$
	public static final String MODULE = "module"; //$NON-NLS-1$
	
	private static final String ERROR = "error"; //$NON-NLS-1$
	private static final String WARNING = "warning"; //$NON-NLS-1$

	private static CSharpCompilerAction instance = new CSharpCompilerAction();

	private static String cSharpCompilerLocation; //$NON-NLS-1$

	// org.eclipse.debug.internal.ui.views.ConsoleView
	//public final static String CONSOLE_ID = "id of the console here"; //$NON-NLS-1$

	/**
	 * Constructs and updates the action.
	 */
	private CSharpCompilerAction() {
		super(NLS.getResourceBundle(), "CompilerAction.", null); //$NON-NLS-1$
		update();

		cSharpCompilerLocation =
			CSharpUIPlugin.getDefault().getPreferenceStore().getString(
				ICSharpPreferenceConstants.C_SHARP_COMPILER_PATH);
		Path path = new Path(""); //$NON-NLS-1$
		if (!path.isValidPath(cSharpCompilerLocation))
			cSharpCompilerLocation = null;
	}

	public static CSharpCompilerAction getInstance() {
		return instance;
	}

	/**
	 * Code called when the action is fired.
	 */
	public void run() {
		IFile fileToCompile = findCurrentCSharpFile();
		if (fileToCompile == null) {
			// should never happen
			System.err.println("Error : no file in the editor");
			// should throw an exception
			return;
		}

		Runtime r = Runtime.getRuntime();

		try {
			
			String command = buildCommand(fileToCompile);
			
			String filePath = fileToCompile.getLocation().toOSString();
			String fileFolderPath =	filePath.substring(0, filePath.length() - fileToCompile.getName().length());

			// clears the console
			printResultInConsole("");
			
			// runs the command
			Process p = r.exec(command, new String[] {}, new File(fileFolderPath));

			// gets the input stream to have the post-compile-time information
			InputStream stream = p.getInputStream();

			// and get the string from it
			String compilerOutput = getStringFromStream(stream);

			if (!compilerOutput.equals("")) // if nesC compiler prints number of errors 
			{
				int errors = Integer.parseInt(compilerOutput); // get the number of errors.
				if (errors == 0)
				{
					printResultInConsole("Promela model generated.");
					executeSpin(r, fileFolderPath);
				}
				else
				{
					stream = p.getErrorStream();

					compilerOutput = getStringFromStream(stream);

					// prints out the information
					printResultInConsole(compilerOutput);
					
				}
			}
			else
			{
				stream = p.getErrorStream();

				compilerOutput = getStringFromStream(stream);

				// prints out the information
				printResultInConsole(compilerOutput);
				
			}
	
//			System.out.println("done?");
		} catch (Exception e) {
			// $$$ should throw the again
//			System.out.println("and there was en error");
//			System.err.println("Problem");
			e.printStackTrace();
		} //catch (CoreException e) {
			// $$$ do something here !
		//}
	}

	protected void executeSpin(Runtime r, String fileFolderPath) throws Exception
	{
		String spin = CSharpUIPlugin.getDefault().getPreferenceStore().getString(
				ICSharpPreferenceConstants.SPIN);
		

		String gcc = CSharpUIPlugin.getDefault().getPreferenceStore().getString(
				ICSharpPreferenceConstants.GCC);
		
		r.exec(spin+" -a promela.pm", new String[] {}, new File(fileFolderPath));
		Thread.sleep(3000); // needed to generate the pan.c file.
		r.exec(" env PATH=/usr/bin "+gcc+" -o pan pan.c -DVECTORSZ=10000 ", new String[] {}, new File(fileFolderPath));
		Thread.sleep(3000); // needed to generate the pan
		
		Process p = r.exec(" "+fileFolderPath+"pan -a ", new String[] {}, new File(fileFolderPath));
		InputStream stream = p.getInputStream();
		String resultOfSpin = getStringFromStream(stream);
		int startIndex = resultOfSpin.indexOf("errors: ");
		int endIndex = resultOfSpin.indexOf("\n", startIndex);
		String numOfErrors = resultOfSpin.substring(startIndex + 8, endIndex); 
		int errors = Integer.parseInt(numOfErrors);
			
		if ( errors > 0)
		{
			continuePrinting("Security property violated, generating the counterexample.");
			Process q = r.exec(" "+spin+" -t "+fileFolderPath+"promela.pm");
			
			File file = new File(fileFolderPath+"output");
			Writer output = new BufferedWriter(new FileWriter(file));
			output.write(getStringFromStream(q.getInputStream()));
			output.close();
			r.exec(" rm "+fileFolderPath+"pan ");
			r.exec(" rm "+fileFolderPath+"pan.h ");
			r.exec(" rm "+fileFolderPath+"pan.c ");
			r.exec(" rm "+fileFolderPath+"pan.m ");
			r.exec(" rm "+fileFolderPath+"pan.b ");
			r.exec(" rm "+fileFolderPath+"pan.t ");
			r.exec(" env PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/root/bin nesc-translator "+fileFolderPath+"/output "+fileFolderPath+"/counterexample.txt", new String[] {}, new File(fileFolderPath));
			r.exec(" rm "+fileFolderPath+"output ");
//			openCounterExample(fileFolderPath, fileFolderPath+"/output");
		}
		else
		{
			continuePrinting("Protocol satisfied the security goals");
		}
	}
	
	protected void openCounterExample(String fileFolderPath, String fileName)
	{
		IEditorRegistry editorRegistry = PlatformUI.getWorkbench().getEditorRegistry();
		IEditorDescriptor defaultEditorDescriptor = editorRegistry.getDefaultEditor(fileName);
		//// || defaultEditorDescriptor.isOpenExternal() is only eclipse 3.x!!!
		if( defaultEditorDescriptor == null){
		defaultEditorDescriptor = editorRegistry.getDefaultEditor("dummy.txt");
		}
		
		try 
		{
			IWorkbenchPage page =
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			ITextEditor editor = getTextEditor();
			IEditorInput editorInput = null;
			if (editor != null) {
				editorInput = editor.getEditorInput();
			}

			if (editorInput instanceof IFileEditorInput)
//			FileEditorInput fileEditorInput = new FileEditorInput(file);
			if (page != null)
			//page.openEditor(fileEditorInput,"org.eclipse.ui.DefaultTextEditor");
				page.openEditor((IFileEditorInput)editorInput, defaultEditorDescriptor.getId() );
			} catch (PartInitException e) {
		}
	}
	
	/**
	 * Finds the file that's currently opened in the C# Text Editor
	 */
	protected IFile findCurrentCSharpFile() {
		ITextEditor editor = getTextEditor();

		IEditorInput editorInput = null;
		if (editor != null) {
			editorInput = editor.getEditorInput();
		}

		if (editorInput instanceof IFileEditorInput)
			return ((IFileEditorInput) editorInput).getFile();

		// if nothing was found, which should never happen
		return null;
	}

	/**
	 * Builds the command for compiling the current file. <br/>
	 * It uses the persistent properties of the file to know the
	 * compile-time arguments.
	 */
	protected String buildCommand(IFile fileToCompile) {
		StringBuffer command = new StringBuffer("env PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/root/bin ");
		
		String theFileName = fileToCompile.getName();
		String configName = theFileName.substring(0, theFileName.indexOf('.'));
		
		
		String TinyTos = CSharpUIPlugin.getDefault().getPreferenceStore().getString(
				ICSharpPreferenceConstants.TINYTOS);
		command.append(" env TOSDIR="+TinyTos);

		String TinyRoot = CSharpUIPlugin.getDefault().getPreferenceStore().getString(
				ICSharpPreferenceConstants.TINYROOT);
		command.append(" env TOSROOT="+TinyRoot);

		command.append(" ncc -o main.exe -g -O0 -board=micasb -pthread -target=pc  -Wall -Wshadow -DDEF_TOS_AM_GROUP=0x7d -Wnesc-all -fnesc-nido-tosnodes=1000 "+configName+".nc -lm");

		/*		
		command.append(" "); //$NON-NLS-1$

		// retrieves the persistent properties
		String outputName = null;
		try {
			outputName =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.outputNameQName);
		} catch (CoreException e) {
		}

		String kindOfTarget = null;
		try {
			kindOfTarget =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.targetQName);
		} catch (CoreException e) {
		}

		String docFileName = null;
		try {
			docFileName =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.docFileNameQName);
		} catch (CoreException e) {
		}

		String refFileNames = null;
		try {
			refFileNames =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.refFileTextQName);
		} catch (CoreException e) {
		}

		String moduleNames = null;
		try {
			moduleNames =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.moduleTextQName);
		} catch (CoreException e) {
		}

		String otherArgs = null;
		try {
			otherArgs =
				fileToCompile.getPersistentProperty(CSharpPropertyPage.otherArgsQName);
		} catch (CoreException e) {
		}

		// give the right name
		command.append("/out:"); //$NON-NLS-1$
		if (outputName != null && !outputName.trim().equals("")) { //$NON-NLS-1$
			// an output name has been specified
			command.append(outputName.trim());
		} else {
			// no output name has been specified: we give the same name as the
			// source code file
			String fileName = fileToCompile.getName();
			// length()-2 to remove the ".cs" extension
			command.append(fileName.substring(0, fileName.length() - 3));
		}
		// and now add the corresponding extension
		if (kindOfTarget != null) {
			if (kindOfTarget.equals(CSharpCompilerAction.EXE)
				|| kindOfTarget.equals(CSharpCompilerAction.WINEXE))
				command.append(".exe"); //$NON-NLS-1$
			else if (kindOfTarget.equals(CSharpCompilerAction.LIBRARY))
				command.append(".dll"); //$NON-NLS-1$
			else if (kindOfTarget.equals(CSharpCompilerAction.MODULE))
				command.append(".netmodule"); //$NON-NLS-1$
		} else {
			command.append(".exe"); //$NON-NLS-1$
		}
		command.append(" "); //$NON-NLS-1$

		if (kindOfTarget != null && !kindOfTarget.equals(CSharpCompilerAction.EXE)) {
			command.append("/t:"); //$NON-NLS-1$
			command.append(kindOfTarget);
			command.append(" "); //$NON-NLS-1$
		}

		if (docFileName != null && !docFileName.trim().equals("")) { //$NON-NLS-1$
			command.append("/doc:"); //$NON-NLS-1$
			command.append(docFileName.trim());
			command.append(" "); //$NON-NLS-1$
		}

		if (refFileNames != null && !refFileNames.trim().equals("")) { //$NON-NLS-1$
			command.append("/r:"); //$NON-NLS-1$
			command.append(refFileNames.trim());
			command.append(" "); //$NON-NLS-1$
		}

		if (moduleNames != null && !moduleNames.trim().equals("")) { //$NON-NLS-1$
			command.append("/addmodule:"); //$NON-NLS-1$
			command.append(moduleNames.trim());
			command.append(" "); //$NON-NLS-1$
		}

		command.append(fileToCompile.getName());
*/
		return command.toString();
	}

	/**
	 * Prints out the string represented by the string buffer
	 */
	protected void printResultInConsole(String output) {
		console = new MessageConsole("My Console", null);
		console.activate();
		ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[]{ console });
		MessageConsoleStream stream = console.newMessageStream();
		stream.println(output);


/*		try {
			IWorkbenchPage page =
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			page.showView(CONSOLE_ID);
			CSharpConsoleView console = (CSharpConsoleView) page.findView(CONSOLE_ID);
			console.showMessage(output);
		} catch (PartInitException e) {
			CSharpUIPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,"", 0, "Console not initialized", e));
		}
		System.out.println(output);
*/
	}

	protected void continuePrinting(String output)
	{
		MessageConsoleStream stream = console.newMessageStream();
		stream.println();
		stream.println(output);		
	}
	
	/**
	 * Creates a string buffer from the given input stream
	 */
	protected String getStringFromStream(InputStream stream) throws IOException {
		StringBuffer buffer = new StringBuffer();
		byte[] b = new byte[100];
		int finished = 0;
		while (finished != -1) {
			finished = stream.read(b);
			if (finished != -1) {
				String current = new String(b, 0, finished);
				buffer.append(current);
				//System.out.print(current);
			}
		}
		return buffer.toString();
	}

	/**
	 * Create markers according to the compiler output
	 */
	protected void createMarkers(String output, IFile file) throws CoreException {
		// first delete all the previous markers
		file.deleteMarkers(IMarker.PROBLEM, false, 0);

		// and then create the new ones
		Vector result = new Vector();
		String fullPath = file.getName();

		StringTokenizer tokenizer = new StringTokenizer(output, "\n"); //$NON-NLS-1$
		while (tokenizer.hasMoreElements()) {
			String current = (String) tokenizer.nextElement();
			if (current.indexOf(fullPath) != -1) {
				String line =
					current.substring(
						current.indexOf(fullPath) + fullPath.length(),
						current.length());
				String errorsLocation = line.substring(1, line.indexOf(":") - 1); //$NON-NLS-1$
				String message = line.substring(line.indexOf(":") + 2, line.length() - 1); //$NON-NLS-1$

				int lineNumber =
					Integer.parseInt(errorsLocation.substring(0, errorsLocation.indexOf(",")));	//$NON-NLS-1$

				IMarker m = file.createMarker(IMarker.PROBLEM);
				m.setAttribute(IMarker.MESSAGE, message);
				if (message.startsWith(ERROR))
					m.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
				else if (message.startsWith(WARNING))
					m.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
				else
					m.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO);
				m.setAttribute(IMarker.LINE_NUMBER, lineNumber);

			}
		}

	}

	public static void setCompilerLocation(String newLocation) {
		cSharpCompilerLocation = newLocation;
	}

}