/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.rules.RuleBasedPartitioner;
import org.eclipse.ui.editors.text.FileDocumentProvider;
import fr.improve.csharp.ui.internal.editor.csharp.CSharpPartitionScanner;

/** 
 * The CSharpDocumentProvider provides the IDocuments used by c sharp editors.
 */

public class CSharpDocumentProvider extends FileDocumentProvider {

	private final static String[] TYPES= new String[] { CSharpPartitionScanner.CSHARP_DOC, CSharpPartitionScanner.CSHARP_ANNOTATION };

	private static CSharpPartitionScanner scanner= null;

	public CSharpDocumentProvider() {
		super();
	}
	
	/* (non-Javadoc)
	 * Method declared on AbstractDocumentProvider
	 */
	 protected IDocument createDocument(Object element) throws CoreException {
		IDocument document= super.createDocument(element);
		if (document != null) {
			IDocumentPartitioner partitioner= createCSharpPartitioner();
			partitioner.connect(document);
			document.setDocumentPartitioner(partitioner);
		}
		return document;
	}
	
	/**
	 * Return a partitioner for .cs files.
	 */
	 private RuleBasedPartitioner createCSharpPartitioner() {
		return new RuleBasedPartitioner(getCSharpPartitionScanner(), TYPES);
	}
	
	/**
	 * Return a scanner for creating c sharp partitions.
	 */
	 private CSharpPartitionScanner getCSharpPartitionScanner() {
		if (scanner == null)
			scanner= new CSharpPartitionScanner();
		return scanner;
	}
}
