/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.DefaultRangeIndicator;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import fr.improve.csharp.ui.internal.CSharpUIPlugin;
import fr.improve.csharp.ui.internal.ICSharpPreferenceConstants;

/**
 * CSharp specific text editor.
 */
public class CSharpEditor extends TextEditor {


	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method extend the 
	 * actions to add those specific to the receiver
	 */
	protected void createActions() {
		super.createActions();
		setAction(
			"ContentAssistProposal",
			new TextOperationAction(
				NLS.getResourceBundle(),
				"ContentAssistProposal.",
				this,
				ISourceViewer.CONTENTASSIST_PROPOSALS));
		//$NON-NLS-1$ //$NON-NLS-2$
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * disposal actions required by the CSharp editor.
	 */
	public void dispose() {
		CSharpEditorEnvironment.disconnect(this);
		super.dispose();
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * revert behavior required by the CSharp editor.
	 */
	public void doRevertToSaved() {
		super.doRevertToSaved();
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * save behavior required by the CSharp editor.
	 */
	public void doSave(IProgressMonitor monitor) {
		super.doSave(monitor);

		// compile or not, according to the user preferences
		if (CSharpUIPlugin.getDefault().getPreferenceStore().getBoolean(ICSharpPreferenceConstants.AUTO_COMPILE)) {
			IAction a = CSharpCompilerAction.getInstance();
			if (a != null)
				a.run();
		}
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * save as behavior required by the CSharp editor.
	 */
	public void doSaveAs() {
		super.doSaveAs();
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs sets the 
	 * input of the outline page after AbstractTextEditor has set input.
	 */
	public void doSetInput(IEditorInput input) throws CoreException {
		super.doSetInput(input);
	}

	/** The <code>CSharpEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method adds any 
	 * CSharpEditor specific entries.
	 */
	public void editorContextMenuAboutToShow(MenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "ContentAssistProposal"); //$NON-NLS-1$
	}

	/* 
	 * Method declared on AbstractTextEditor
	 */
	protected void initializeEditor() {

		CSharpEditorEnvironment.connect(this);

		setSourceViewerConfiguration(new CSharpSourceViewerConfiguration());
		setRangeIndicator(new DefaultRangeIndicator());
		setEditorContextMenuId("#CSharpEditorContext"); //$NON-NLS-1$
		setRulerContextMenuId("#CSharpRulerContext"); //$NON-NLS-1$
	}
}