/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;


import org.eclipse.jface.text.rules.RuleBasedScanner;
import fr.improve.csharp.ui.internal.editor.csharp.CSharpCodeScanner;
import fr.improve.csharp.ui.internal.editor.doc.CSharpDocScanner;
import fr.improve.csharp.ui.internal.editor.utils.CSharpColorProvider;

/** 
 * The CSharpEditorEnvironment maintains singletons used by the c sharp editor
 */
public class CSharpEditorEnvironment {

	private static CSharpColorProvider colorProvider;
	private static CSharpCodeScanner codeScanner;
	private static CSharpDocScanner docScanner;

	private static int referenceCounter= 0;

	/**
	 * A connection has occured - initialize the receiver if it is the first activation.
	 */
	public static void connect(Object client) {
		if (++referenceCounter == 1) {
			colorProvider= new CSharpColorProvider();
			codeScanner= new CSharpCodeScanner(colorProvider);
			docScanner= new CSharpDocScanner(colorProvider);
		}
	}
	
	/**
	 * A disconnection has occured - clear the receiver if it is the last deactivation.
	 */
	 public static void disconnect(Object client) {
		if (--referenceCounter == 0) {
			codeScanner= null;
			docScanner= null;
			colorProvider.dispose();
			colorProvider= null;
		}
	}
	
	/**
	 * Returns the singleton scanner.
	 */
	 public static RuleBasedScanner getCSharpCodeScanner() {
		return codeScanner;
	}
	
	/**
	 * Returns the singleton color provider.
	 */
	 public static CSharpColorProvider getCSharpColorProvider() {
		return colorProvider;
	}
	
	/**
	 * Returns the singleton document scanner.
	 */
	 public static RuleBasedScanner getCSharpDocScanner() {
		return docScanner;
	}
}
