/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor;


import org.eclipse.swt.graphics.RGB;
import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.*;
import org.eclipse.jface.text.presentation.*;
import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.source.*;
import fr.improve.csharp.ui.internal.editor.csharp.*;
import fr.improve.csharp.ui.internal.editor.doc.CSharpDocCompletionProcessor;
import fr.improve.csharp.ui.internal.editor.utils.CSharpColorProvider;

/**
 * Configuration for an <code>SourceViewer</code> which shows c sharp code.
 */
public class CSharpSourceViewerConfiguration extends SourceViewerConfiguration {

	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IAutoIndentStrategy getAutoIndentStrategy(ISourceViewer sourceViewer, String contentType) {
		return (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? new CSharpAutoIndentStrategy() : new DefaultAutoIndentStrategy());
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, CSharpPartitionScanner.CSHARP_DOC, CSharpPartitionScanner.CSHARP_ANNOTATION };
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		ContentAssistant assistant= new ContentAssistant();
		assistant.setContentAssistProcessor(new CSharpCompletionProcessor(), IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(new CSharpDocCompletionProcessor(), CSharpPartitionScanner.CSHARP_DOC);

		assistant.enableAutoActivation(true);
		assistant.setAutoActivationDelay(500);
		assistant.setProposalPopupOrientation(assistant.PROPOSAL_OVERLAY);
		assistant.setContextInformationPopupOrientation(assistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(CSharpEditorEnvironment.getCSharpColorProvider().getColor(new RGB(150, 150, 0)));

		return assistant;
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String getDefaultPrefix(ISourceViewer sourceViewer, String contentType) {
		return (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? "//" : null); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public ITextDoubleClickStrategy getDoubleClickStrategy(ISourceViewer sourceViewer, String contentType) {
		return new CSharpDoubleClickSelector();
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String[] getIndentPrefixes(ISourceViewer sourceViewer, String contentType) {
		return new String[] { "\t", "    " }; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {

		PresentationReconciler reconciler= new PresentationReconciler();

		// rule for default text
		RuleBasedDamagerRepairer dr= new RuleBasedDamagerRepairer(CSharpEditorEnvironment.getCSharpCodeScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		// rule for multiline comments
		// We jsut need a scanner that does nothing but returns a token with the corrresponding text attributes
		RuleBasedScanner multiLineScanner = new RuleBasedScanner();
		multiLineScanner.setDefaultReturnToken(new Token(new TextAttribute(CSharpEditorEnvironment.getCSharpColorProvider().getColor(CSharpColorProvider.MULTI_LINE_COMMENT))));
		dr= new RuleBasedDamagerRepairer(multiLineScanner);
		reconciler.setDamager(dr, CSharpPartitionScanner.CSHARP_ANNOTATION);
		reconciler.setRepairer(dr, CSharpPartitionScanner.CSHARP_ANNOTATION);

		// rule for c sharp comments for documentation
		dr= new RuleBasedDamagerRepairer(CSharpEditorEnvironment.getCSharpDocScanner());
		reconciler.setDamager(dr, CSharpPartitionScanner.CSHARP_DOC);
		reconciler.setRepairer(dr, CSharpPartitionScanner.CSHARP_DOC);
		
		return reconciler;
	}
	
	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return 4;
	}
	
}
