/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor.csharp;


import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;
import fr.improve.csharp.ui.internal.editor.utils.CSharpColorProvider;
import fr.improve.csharp.ui.internal.editor.utils.CSharpWhitespaceDetector;
import fr.improve.csharp.ui.internal.editor.utils.CSharpWordDetector;

/**
 * A C Sharp code scanner.
 */
public class CSharpCodeScanner 
				extends RuleBasedScanner 
				implements ICSharpLanguageWords {


	private TextAttribute fComment;
	private TextAttribute fKeyword;
	private TextAttribute fType;
	private TextAttribute fString;
	private CSharpColorProvider fColorProvider;

	/**
	 * Creates a C Sharp code scanner
	 */
	public CSharpCodeScanner(CSharpColorProvider provider) {

		IToken keyword= new Token(new TextAttribute(provider.getColor(CSharpColorProvider.KEYWORD), provider.getColor(CSharpColorProvider.BACKGROUND), SWT.BOLD));
		IToken type= new Token(new TextAttribute(provider.getColor(CSharpColorProvider.TYPE), provider.getColor(CSharpColorProvider.BACKGROUND), SWT.BOLD));
		IToken string= new Token(new TextAttribute(provider.getColor(CSharpColorProvider.STRING)));
		IToken comment= new Token(new TextAttribute(provider.getColor(CSharpColorProvider.SINGLE_LINE_COMMENT)));
		IToken other= new Token(new TextAttribute(provider.getColor(CSharpColorProvider.DEFAULT)));

		setDefaultReturnToken(other);
		
		List rules= new ArrayList();

		// Add rule for single line comments.
		rules.add(new EndOfLineRule("//", comment)); //$NON-NLS-1$

		// Add rule for strings and character constants.
		rules.add(new SingleLineRule("\"", "\"", string, '\\')); //$NON-NLS-2$ //$NON-NLS-1$
		rules.add(new SingleLineRule("'", "'", string, '\\')); //$NON-NLS-2$ //$NON-NLS-1$

		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new CSharpWhitespaceDetector()));

		// Add word rule for keywords, types, and constants.
		WordRule wordRule= new WordRule(new CSharpWordDetector(), other);
		for (int i= 0; i < keywords.length; i++)
			wordRule.addWord(keywords[i], keyword);
		for (int i= 0; i < types.length; i++)
			wordRule.addWord(types[i], type);
		for (int i= 0; i < constants.length; i++)
			wordRule.addWord(constants[i], type);
		rules.add(wordRule);

		IRule[] result= new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}
