/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor.doc;

import java.util.*;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.*;

import fr.improve.csharp.ui.internal.editor.csharp.ICSharpLanguageWords;
import fr.improve.csharp.ui.internal.editor.utils.PieceOfWord;

/**
 * C Sharp doc completion processor.
 */
public class CSharpDocCompletionProcessor
	implements IContentAssistProcessor, ICSharpLanguageWords {

	protected Vector proposalVector = new Vector();

	protected int currentIndex = 0;

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public ICompletionProposal[] computeCompletionProposals(
		ITextViewer viewer,
		int documentOffset) {

		PieceOfWord word = new PieceOfWord(viewer, documentOffset);

		// iterate over all the CSharpDocWords categories
		for (int y = 0; y < cSharpDocWords.length; y++) {
			if (cSharpDocWords[y].startsWith(word.getString()))
				proposalVector.add(cSharpDocWords[y]);
		}

		return turnProposalVectorIntoAdaptedArray(word);
	}

	/*
	 * Turns the vector into an Array of ICompletionProposal objects
	 */
	protected ICompletionProposal[] turnProposalVectorIntoAdaptedArray(PieceOfWord word) {
		ICompletionProposal[] result = new ICompletionProposal[proposalVector.size()];
		int index = 0;

		for (Iterator i = proposalVector.iterator(); i.hasNext();) {
			String keyWord = (String) i.next();
			result[index] =
				new CompletionProposal(
					keyWord,
					word.getOffset(),
					word.getString().length(),
					keyWord.length());
			index++;
		}
		//System.out.println("result : " + result.length);
		proposalVector.removeAllElements();
		return result;
	}

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public IContextInformation[] computeContextInformation(
		ITextViewer viewer,
		int documentOffset) {
		return null;
	}

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public char[] getCompletionProposalAutoActivationCharacters() {
		return null;
	}

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}

	/* (non-Javadoc)
	 * Method declared on IContentAssistProcessor
	 */
	public String getErrorMessage() {
		return null;
	}
}