/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor.doc;

import java.util.*;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.*;
import org.eclipse.swt.SWT;
import fr.improve.csharp.ui.internal.editor.csharp.ICSharpLanguageWords;
import fr.improve.csharp.ui.internal.editor.utils.*;

/**
 * A rule based C Sharp Doc scanner.
 */
public class CSharpDocScanner
	extends RuleBasedScanner
	implements ICSharpLanguageWords {

	/**
	 * A key word detector.
	 */
	static class CSharpDocKeyWordDetector implements IWordDetector {
		public boolean isWordStart(char c) {
			return c=='<';
		}

		public boolean isWordPart(char c) {
			return Character.isLetter(c) || c=='/' || c=='>';
		}
	};


	/**
	 * Create a new c sharp doc scanner.
	 */
	public CSharpDocScanner(CSharpColorProvider provider) {
		super();
		
		IToken defaultToken =
			new Token(new TextAttribute(provider.getColor(CSharpColorProvider.CSHARP_DOC_DEFAULT)));
		IToken keyword =
			new Token(new TextAttribute(provider.getColor(CSharpColorProvider.CSHARP_DOC_KEYWORD), provider.getColor(CSharpColorProvider.BACKGROUND), SWT.BOLD));
		IToken tag =
			new Token(new TextAttribute(provider.getColor(CSharpColorProvider.CSHARP_DOC_TAG)));

		setDefaultReturnToken(defaultToken);

		List list = new ArrayList();

		// Add generic whitespace rule.
		list.add(new WhitespaceRule(new CSharpWhitespaceDetector()));

		// Add word rule for keywords.
		WordRule tagRule = new WordRule(new CSharpDocKeyWordDetector(), tag);
		for (int i = 0; i < cSharpDocWords.length; i++) {
			tagRule.addWord("<" + cSharpDocWords[i] + ">", keyword); //$NON-NLS-1$ //$NON-NLS-2$
			tagRule.addWord("</" + cSharpDocWords[i] + ">", keyword); //$NON-NLS-1$ //$NON-NLS-2$
		}
		list.add(tagRule);


		IRule[] result = new IRule[list.size()];
		list.toArray(result);
		setRules(result);
	}
}