/*
 * (c) Copyright Improve S.A., 2002.
 * All Rights Reserved.
 */
package fr.improve.csharp.ui.internal.editor.utils;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;

/**
 * Class used to compute, for a given resource, its path relative to another
 * resource.
 */

public class RelativePath {

	/**
	 * Computes the relative path.
	 * 
	 * @param resource: the resource for which we want the relative path
	 * @param origin: the resource which the path should be made relative to
	 * @return the string corresponding to the relative path
	 */
	public static String computeRelativePath(
		IResource resource,
		IResource origin) {
		StringBuffer returnPath = new StringBuffer();

		IPath resPath = resource.getProjectRelativePath();
		IPath originPath = origin.getProjectRelativePath();

		if (resPath.segmentCount() == originPath.segmentCount())
			return resource.getName();

		else {
			int common = resPath.matchingFirstSegments(originPath);
			int resSegmentCount = resPath.segmentCount();
			int originSegmentCount = originPath.segmentCount();

			int goDown = originSegmentCount - common - 1;
			for (int i = 0; i < goDown; i++)
				returnPath.append("..\\");
			returnPath.append(resPath.removeFirstSegments(common).toOSString());
		}

		return returnPath.toString();
	}

}